<?php

namespace App\Controller;

use SplFileInfo;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Finder\Finder;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Process\Process;
use Symfony\Component\Routing\Attribute\Route;

final class MainController extends AbstractController
{
    #[Route('/', 'app_root')]
    public function root(): Response
    {
        return $this->index('.index');
    }

    #[Route('/sync', 'app_sync')]
    public function sync(): Response
    {
        $gitDir = $this->getParameter('kernel.project_dir') . '/var/data';

        (new Process(['git', '-C', $gitDir, 'fetch']))->mustRun();
        (new Process(['git', '-C', $gitDir, 'reset', '--hard', 'FETCH_HEAD']))->mustRun();

        return new Response('Updated.');
    }

    #[Route('/{path}', 'app_index', ['path' => '.+'])]
    public function index(string $path): Response
    {
        $gitDir = $this->getParameter('kernel.project_dir') . '/var/data';


        // ---------------------------------------------- Get file contents ---

        $fullPath = sprintf(
            '%s/var/data/%s.md',
            $this->getParameter('kernel.project_dir'),
            $path,
        );

        $fs = new Filesystem();
        $contents = $fs->exists($fullPath) ? $fs->readFile($fullPath) : null;


        // -------------------------------------------------- Get data tree ---

        $finder = new Finder();
        $finder->files()->in($this->getParameter('kernel.project_dir') . '/var/data/');
        if (!$finder->hasResults()) throw $this->createNotFoundException();


        // Convert flat list into a nested tree
        $tree = [];
        foreach ($finder as $file) {
            $current = &$tree;

            // Dunno why it is undefined, it works
            foreach (explode('/', $file->getRelativePath()) as $part) {
                if ($part === '') break;

                if (!isset($current[$part])) {
                    $current[$part] = [];
                }
                $current = &$current[$part];
            }

            $current[$file->getFilenameWithoutExtension()] = [];
        }

        return $this->render('main/index.html.twig', [
            'path'     => $path,
            'contents' => $contents,
            'tree'     => $tree,
            'version'  => (new Process(['git', '-C', $gitDir, 'log', '-1', '--oneline']))->mustRun()->getOutput(),
        ]);
    }
}
